<div class="container-fluid">
  <div class="row">
    <div class="col col-md-6 offset-md-3">
      <div class="card border-danger mt-5">
        <h5 class="card-header"><i class="mdi mdi-alert"></i> Server Error</h5>
        <div class="card-body">
          <strong>{{ type_ }}</strong>: {{ exception }}
        </div>
      </div>
    </div>
  </div>
</div>
