{% extends "generic/object.html" %}

{% block content %}
  <div class="row mb-3">
    <div class="col col-md-6">
      <div class="card">
        <h5 class="card-header">Server</h5>
        <div class="card-body">
          <table class="table table-hover attr-table">
            <tr>
              <th scope="row">Name</th>
              <td>{{ object.name }}</td>
            </tr>
            <tr>
              <th scope="row">Server URL</th>
              <td>{{ object.server_url }}</td>
            </tr>
            <tr>
              <th scope="row">DHCPv6</th>
              <td>{% checkmark object.dhcp6 %}</td>
            </tr>
            <tr>
              <th scope="row">DHCPv4</th>
              <td>{% checkmark object.dhcp4 %}</td>
            </tr>
          </table>
        </div>
      </div>
    </div>
    <div class="col col-md-6">
      {% include "inc/panels/tags.html" %}
    </div>
  </div>
{% endblock content %}
